"""
Kibana CVE lookup tool

This module send a request to the NVD CVE database and lists all the CVEs the Kibana version running on the host
might be vulnerable to

Contains:
- Vuln class for performing the test
- run() function as an entry point for running the test
"""

import http
from http import HTTPStatus
import requests
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint
from json import dumps

__TESTLABEL__ = "Kibana CVE lookup"


class Vuln:
    """
    This class prints all CVEs that the host might be vulnerable to and adds them to the JSON output
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response
        self.helpers.print_header(__TESTLABEL__)


    def _get_cvss_score(self, cve_data: dict) -> str:
        """
        This method extracts the CVSS score from the JSON output provided by the NVD API
        """
        versions = ["31", "30", "2"]

        for version in versions:
            if cvss_data := cve_data.get("cve", {}).get("metrics", {}).get(f"cvssMetricV{version}", []):
                for cvss_metric in cvss_data:
                    if "nvd" in cvss_metric.get("source", ""):
                        return cvss_metric.get("cvssData", {}).get("baseScore", "?")

        return "?"


    def _print_cve(self, cve_list: list):
        """
        This method goes through the provided list of CVEs and prints them out to the terminal and also adds them to the JSON
        output
        """
        for cve in cve_list:
            cve_id = cve["cve"]["id"]
            cvss_score = self._get_cvss_score(cve)
            msg = f"The host may be vulnerable to {cve_id}"
            link = f"https://nvd.nist.gov/vuln/detail/{cve_id}"
            cvss = f"CVSS Score: {cvss_score}"
            ptprint(f"{msg:<50}{link:<52}{cvss}","VULN", not self.args.json, indent=8)

            kbn_node_key = self.helpers.check_node("swKibana")
            if kbn_node_key:
                self.ptjsonlib.add_vulnerability(f"PTV-{cve_id}", node_key=kbn_node_key)
            else:
                self.ptjsonlib.add_vulnerability(f"PTV-{cve_id}")


    def _get_kbn_version(self) -> str:
        try:
            response = self.http_client.send_request(url=self.args.url+"api/status", method="GET", headers=self.args.headers, follow_redirects=False)
        except requests.exceptions.RequestException as error_msg:
            self.ptjsonlib.end_error(f"Error retrieving response", details=error_msg, condition=self.args.json)

        if response.status_code != HTTPStatus.OK:
            ptprint("Could not reach /api/status endpoint", "OK", not self.args.json, indent=4)
            ptprint(f"Received response:\n{dumps(response.json(), indent=4)}", "ADDITIONS",
                    self.args.verbose, indent=4, colortext=True)
            return ""

        try:
            data = response.json()
        except ValueError as error_msg:
            ptprint(f"Error communicating with API: {error_msg}", "ERROR", not self.args.json, indent=4)
            return ""

        version = data.get("version", {})

        if type(version) == dict:
            version = version.get("number", "")


        return version

    def run(self) -> None:
        """
        Executes the Kibana CVE lookup

        The test first sends a request to the NVD CVE API (https://nvd.nist.gov/developers/vulnerabilities) to look for vulnerabilities in the host's Kibana version.
        It then goes through the response and prints the vulnerabilities with the _print_cve() method

        If we get a different HTTP response other than 200 OK, the test exits
        """
        kbn_version = self._get_kbn_version()

        if not kbn_version:
            ptprint(f"Could not get Kibana version", "ERROR", not self.args.json, indent=4)
            return

        nvd_url = f"https://services.nvd.nist.gov/rest/json/cves/2.0?cpeName=cpe:2.3:a:elastic:Kibana:{kbn_version}&isVulnerable"

        try:
            response = self.http_client.send_request(method="GET", url=nvd_url)
        except requests.exceptions.RequestException as e:
            self.ptjsonlib.end_error(f"Error retrieving response from NVD database:", details=e,
                                     condition=self.args.json)

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Error retrieving response from NVD database. Received response: {response.status_code} {response.text}",
                    "ADDITIONS", self.args.verbose, indent=4, colortext=True)
            return

        response = response.json()
        cve_list = response.get("vulnerabilities", [])

        if cve_list:
            ptprint(f"Identified {response['totalResults']} possible vulnerabilities in Kibana {kbn_version}", "VULN",
                    not self.args.json, indent=4)
            self._print_cve(cve_list)
        else:
            ptprint(f"Could not identify any publicly known vulnerabilities in Kibana {kbn_version}",
                    "OK",
                    not self.args.json, indent=4)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the Vuln test"""
    Vuln(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Kibana availability test

This module implements a test that checks if the provided host is running Kibana or not

Contains:
- IsKibana to perform the availability test
- run() function as an entry point for running the test
"""

from requests import Response
from http import HTTPStatus
from xml.etree.ElementPath import prepare_parent
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Kibana availability test"


class IsKibana:
    """
    This class checks to see if a host is running Kibana by looking for the 'kbn' or 'kibana' string in the servers response
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _check_headers(self, headers: object, value: str) -> bool:
        """
        :return: True if the response headers contain the provided value.
        """
        for current in headers:
            if value in current.lower():
                return True

        return False

    def run(self) -> None:
        """
        Executes the Kibana availability test

        The method looks for the 'kibana' and 'kbn' strings in the reponse body and headers

        If the response body/headers contain the string '̈́kibana' the method prints a message that the host is running Kibana.
        If the response body/headers contain the string 'kbn' the method prints a message that the host might be running Kibana.

        Otherwise, exits with an error
        """

        response = self.base_response

        certain = ["kibana" in response.text.lower(), self._check_headers(response.headers.values(), "kibana")]
        probable = ["kbn" in response.text.lower(), self._check_headers(response.headers.keys(), "kbn")]

        if any(certain) :
            ptprint("The host is running Kibana", "INFO", not self.args.json, indent=4)
        elif any(probable):
            ptprint("The host might be running Kibana", "INFO", not self.args.json, indent=4)
        else:
            self.ptjsonlib.end_error("The host is not running kibana", self.args.json)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the IsKibana test"""
    IsKibana(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Kibana HTTP/S test

This module tests if a Kibana instance is running on HTTPS or HTTP
"""

import http
from http import HTTPStatus
from http.client import responses
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Kibana HTTP/S test"


class HttpTest:
    """
    This class tests to see if the host has Kibana running on HTTP or HTTPS
    """

    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def run(self) -> None:
        """
        Executes the Kibana HTTP/S test

        Checks to see if the URL we have sent the initial request contains http:// or https:// in the URL
        """

        if "https://" in self.args.url:
            ptprint(f"The host is running on HTTPS", "OK", not self.args.json, indent=4)
            return

        ptprint(f"The host is running on HTTP", "VULN", not self.args.json, indent=4)
        self.ptjsonlib.add_vulnerability("PTV-KIBANA-MISC-HTTP")


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the HTTP/S test"""
    HttpTest(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Kibana software enumeration module

This module enumerates software running on a Kibana instance
"""

import http
from http import HTTPStatus
from http.client import responses

import requests.exceptions
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint
from json import dumps

__TESTLABEL__ = "Kibana software enumeration"


class SwTest:
    """
    This class enumerates software running on a Kibana host
    """

    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _add_to_json(self, properties: dict, node_name) -> None:
        """
        This method adds a JSON node to the JSON output with the specified properties and node_name
        """
        json_node = self.ptjsonlib.create_node_object(node_name, properties=properties)
        self.ptjsonlib.add_node(json_node)


    def _list_plugins(self, data: dict) -> None:
        """
        This method retrieves available plugins and their status, prints them out and adds them to the JSON output
        """
        status_object = data.get("status", {})
        plugins = status_object.get("statuses", status_object.get("plugins", {}))

        if plugins and type(plugins) == list:
            id_key = "name" if plugins[0].get("name", "") else "id"
            ptprint(f"Found plugins:", "INFO", not self.args.json, indent=4)
            for plugin in plugins:
                status = plugin.get('state', 'unknown')
                name = plugin.get(id_key, "").replace("plugin:", "")
                if "core:" in name:
                    continue
                self._add_to_json({"name": name, "status": status}, "swPlugin")
                ptprint(f"Name: {name:<35} Status: {status:<15}", "INFO", not self.args.json, indent=8)

        elif plugins and type(plugins) == dict:
            ptprint(f"Found plugins:", "INFO", not self.args.json, indent=4)
            for plugin in plugins.keys():
                status = plugins.get(plugin, {}).get("level", "unknown")
                self._add_to_json({"name": plugin, "status": status}, "swPlugin")
                ptprint(f"Name: {plugin:<35} Status: {status:<15}", "INFO", not self.args.json, indent=8)

        else:
            ptprint(f"Could not retrieve available plugins", "OK", not self.args.json, indent=4)


    def _list_core_plugins(self, data: dict) -> None:
        """
        This method retrieves available core plugins and their status, prints them out and adds them to the JSON output
        """
        if cores := data.get("status", {}).get("core", {}):
            for core in cores:
                name = core
                status = cores.get(core, {}).get("level", "unknown")
                self._add_to_json({"id": name, "status": status}, "swCorePlugin")
                ptprint(f"Name: {name:<35} Status: {status:<15}", "INFO",
                        not self.args.json, indent=8)
            return

        id_key = "name" if data.get("status", {}).get("statuses", [{}])[0].get("name", "") else "id"
        cores = [core for core in data.get("status", {}).get("statuses", {}) if "core" in core.get(id_key, "")]

        if cores:
            ptprint(f"Found core plugins:", "INFO", not self.args.json, indent=4)
            for core in cores:
                name = core.get(id_key, "").replace("core:", "")
                status = core.get('state', 'unknown')
                self._add_to_json({"id": name, "status": status}, "swCorePlugin")
                ptprint(f"Name: {name:<35} Status: {status:<15}", "INFO",
                        not self.args.json, indent=8)

        else:
            ptprint(f"Could not retrieve available core plugins", "OK", not self.args.json, indent=4)


    def _list_os_properties(self, data: dict) -> None:
        """
        This method retrieves OS properties, prints them out and adds them to the JSON output
        """
        os_properties = data.get("metrics", {}).get("os", {})

        platform = os_properties.get('platform', '')
        platform_release = os_properties.get('platformRelease', '')
        distro = os_properties.get('distro', '')
        distro_release = os_properties.get('distroRelease', '')

        if not any([platform, platform_release, distro_release, distro]):
            ptprint(f"Could not retrieve OS properties", "OK", not self.args.json, indent=4)
            return

        ptprint(f"OS properties:", "INFO", not self.args.json, indent=4)
        ptprint(f"{'Platform:':<41} {platform}", "INFO",
                not self.args.json, indent=8) if platform else None
        ptprint(f"{'Release:':<41} {platform_release}", "INFO",
                not self.args.json, indent=8) if platform_release else None
        ptprint(f"{'Distro:':<41} {distro}", "INFO",
                not self.args.json, indent=8) if distro else None
        ptprint(f"{'Distro release:':<41} {distro_release}", "INFO",
                not self.args.json, indent=8) if distro_release else None

        self._add_to_json({
                "platform": os_properties.get('platform', ''),
                "release": os_properties.get('platformRelease', ''),
                "distro": os_properties.get('distro', ''),
                "distroRelease": os_properties.get('distroRelease', '')
             },
        "osProperties")


    def _list_kbn_version(self, data: dict) -> None:
        """
        This method retrieves the Kibana version, prints it out and adds it to the JSON output
        """
        version = data.get("version", {})

        if type(version) == dict:
            version = version.get("number", "")

        if version:
            self._add_to_json({"name": data.get("name", ""), "version": version}, "swKibana")
            ptprint(f"{'Kibana version:':<45} {version}", "INFO", not self.args.json, indent=4)
        else:
            ptprint("Could not enumerate version", "OK", not self.args.json, indent=4)


    def run(self) -> None:
        """
        Executes the Kibana software enumeration module

        This method first retrieves available plugins, core plugins, os properties and the Kibana instance version.
        It does so by sending an HTTP GET request to the /api/status output and then going through the JSON response.

        If successful, it prints the information out and adds it to the JSON output.
        """
        try:
            response = self.http_client.send_request(url=self.args.url+"api/status", method="GET", headers=self.args.headers, follow_redirects=False)
        except requests.exceptions.RequestException as error_msg:
            self.ptjsonlib.end_error(f"Error retrieving response", details=error_msg, condition=self.args.json)

        if response.status_code != HTTPStatus.OK:
            ptprint("Could not reach /api/status endpoint", "OK", not self.args.json, indent=4)
            ptprint(f"Received response:\n{dumps(response.json(), indent=4)}", "ADDITIONS",
                    self.args.verbose, indent=4, colortext=True)
            return

        try:
            data = response.json()
        except ValueError as error_msg:
            ptprint(f"Error communicating with API: {error_msg}", "ERROR", not self.args.json, indent=4)
            return


        self._list_kbn_version(data)
        self._list_core_plugins(data)
        self._list_plugins(data)
        self._list_os_properties(data)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the SW enumeration module"""
    SwTest(args, ptjsonlib, helpers, http_client, base_response).run()
